'''
Prolog mailbox parser class
@author: Peeter Jyrviste
'''
import AbstractMailboxParser, BodyParser
import mailbox, time

# writes attachments info into given file
def getAttachments(message, fileOut):
    "String file, open file"
    for part in message.walk():
        filename = part.get_filename()
        if not filename:
            filename = part.get_param('name', None, 'content-type')
        if filename != None:
            fileOut.write("\t\t['Attachment:'," + repr(escapeHyphens(filename)) + "],\n")

# escapes hyphens in a string
def escapeHyphens(string):
    result = "\\'".join(string.split("'"))
    return result

# separates tokens by commas
def separateByCommas(string):
    tokens = string.split(',')
    for index, part in enumerate(tokens):
        part = part.strip()
        tokens[index] = part
    return tokens

# outputs header info into given file, 
# given header name, message object, output file name 
# and whether it should be separated by commas
def getHeaderInfo(header, message, fileOut, separationEnabled):
    if message.has_key(header):
        if(separationEnabled):
            myList = separateByCommas(escapeHyphens(message[header]))
            for i in myList:
                fileOut.write("\t\t['" + header + ":','" + i + "'],\n")
        else:
            fileOut.write("\t\t['" + header + ":','" + escapeHyphens(message[header]) + "'],\n")

class PrologParser(AbstractMailboxParser.AbstractMailboxParser):  
    def __init__(self, mailbox, outputFile):
        self.mailbox = mailbox
        self.outputFile = outputFile
       
    "Email transformer for Prolog output, extends emailTransformer"    
    def displayMail(self):
        print 'Saving statements to', self.outputFile
        print 'Please wait...'
        start = time.clock()
        myMbox = mailbox.mbox(self.mailbox)
        fileOut = file(self.outputFile, 'w')
        bodyParser = BodyParser.BodyParser()      
        elements = len(myMbox)
        fileOut.write('mbox([\n')
        for message in myMbox:
            fileOut.write('\tmessage([\n')
            fileOut.write("\t\t" + str(bodyParser.processParas(message)) + ",\n")
            getHeaderInfo('Message-ID', message, fileOut, True)
            getHeaderInfo('In-Reply-To', message, fileOut, True)
            getHeaderInfo('References', message, fileOut, True)
            getHeaderInfo('Subject', message, fileOut, False)
            getHeaderInfo('Comments', message, fileOut, False)
            getHeaderInfo('Keywords', message, fileOut, True)
            getHeaderInfo('From', message, fileOut, True)
            getHeaderInfo('Sender', message, fileOut, True)
            getHeaderInfo('Reply-To', message, fileOut, True)
            getHeaderInfo('To', message, fileOut, True)
            getHeaderInfo('Cc', message, fileOut, True)
            getHeaderInfo('Bcc', message, fileOut, True)
            if message.has_key('Received'):
                received_lines = message.get_all('Received')
                for line in received_lines:
                    fileOut.write("\t\t['Received:','" + escapeHyphens(line) + "'],\n")
            getAttachments(message, fileOut)
            if message.has_key('Date'):
                fileOut.write("\t\t['Date:','" + escapeHyphens(message['date']) + "']\n")
                elements = elements - 1
            if elements > 0:
                fileOut.write('\t]),\n')
            else:
                fileOut.write('\t])\n')
        fileOut.write(']).')
        fileOut.close()
        end = time.clock()
        print 'Prolog statements saved.'
        print "Time elapsed:", round(end - start, 3), "seconds"
